<?php

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit();
}

use Sendeasy\SE_SOAP_Client;

if (class_exists('SE_Sendeasy_WC_Settings_Page', false)) {
    return new SE_Sendeasy_WC_Settings_Page();
}

/**
 * SE_Sendeasy_WC_Settings_Page class.
 *
 * Extends the WooCommerce abstract class WC_Settings_Page
 *
 * @see https://github.com/woocommerce/woocommerce/blob/master/includes/admin/settings/class-wc-settings-page.php
 */
class SE_Sendeasy_WC_Settings_Page extends WC_Settings_Page
{
    /**
     * Constructor.
     */
    public function __construct()
    {

        $this->id = 'se_sendeasy';
        $this->label = __('SendEasy', 'sendeasy');

        add_filter('woocommerce_settings_tabs_array', [$this, 'add_settings_page'], 20);
        add_action('woocommerce_settings_' . $this->id, [$this, 'output']);
        add_action('woocommerce_settings_save_' . $this->id, [$this, 'save']);
        add_action('woocommerce_sections_' . $this->id, [$this, 'output_sections']);
    }

    /**
     * Get the SendEasy settings sections
     *
     * @return array
     */
    public function get_sections(): array
    {

        $sections = [
            '' => __('Connection', 'sendeasy'),
            'se_customer_sms_section' => __('Customer SMS', 'sendeasy'),
            'se_admin_sms_section' => __('Admin SMS', 'sendeasy')
        ];

        return apply_filters('woocommerce_get_sections_' . $this->id, $sections);
    }


    /**
     * Get the SendEasy settings array
     *
     * @access public
     * @param string $current_section
     * @return array
     */
    public function get_settings(string $current_section = ''): array
    {

        if ('se_customer_sms_section' == $current_section) {
            /**
             * Filter the SendEasy customer SMS section settings
             *
             * @param array $settings
             */
            $settings = apply_filters('se_customer_sms_section_settings', [
                [
                    'name' => __('Customer SMS notifications', 'sendeasy'),
                    'type' => 'title',
                    'id' => 'se_sendeasy_customer_sms_options'
                ],
                [
                    'type' => 'multiselect',
                    'id' => 'se_enabled_customer_sms_notifications',
                    'name' => __('Order statuses to send SMS notifications for', 'sendeasy'),
                    'options'  => [
                        'pending' => __('Pending payment', 'sendeasy'),
                        'processing' => __('Processing', 'sendeasy'),
                        'on-hold' => __('On hold', 'sendeasy'),
                        'completed' => __('Completed', 'sendeasy'),
                        'cancelled' => __('Cancelled', 'sendeasy'),
                        'failed' => __('Failed', 'sendeasy'),
                        'refunded' => __('Refunded', 'sendeasy'),
                    ],
                    'class' => 'wc-enhanced-select',
                    'desc_tip' => __('Add the statuses that you want to send SMS updates for', 'sendeasy'),
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_customer_sms_template_default',
                    'name' => __('Default customer SMS template', 'sendeasy'),
                    'desc' => sprintf(
                        '%s
                                <ul>
                                    <li><code>{shop_name}</code>: <i>%s</i></li>
                                    <li><code>{order_id}</code>: <i>%s</i></li>
                                    <li><code>{order_status}</code>: <i>%s</i></li>
                                    <li><code>{order_total}</code>: <i>%s</i></li>
                                    <li><code>{billing_first_name}</code>: <i>%s</i></li>
                                    <li><code>{billing_last_name}</code>: <i>%s</i></li>
                                    <li><code>{shipping_first_name}</code>: <i>%s</i></li>
                                    <li><code>{shipping_last_name}</code>: <i>%s</i></li>
                                    <li><code>{shipping_method}</code>: <i>%s</i></li>
                                </ul>
                                <b><em>%s</em></b>',
                        __('Use these tags to customise your message:', 'sendeasy'),
                        __('Your WordPress site title.', 'sendeasy'),
                        __("Customer's order number.", 'sendeasy'),
                        __("Customer's order status.", 'sendeasy'),
                        __("Total cost of the customer's order.", 'sendeasy'),
                        __("Customer's first name.", 'sendeasy'),
                        __("Customer's last name.", 'sendeasy'),
                        __("Recipient's first name, if shipping to a different address.", 'sendeasy'),
                        __("Recipient's last name, if shipping to a different address.", 'sendeasy'),
                        __('Shipping method chosen for the order.', 'sendeasy'),
                        __('Remember that SMS messages must be limited to 160 characters or less.', 'sendeasy')
                    ),
                    'default' => sprintf(
                        __('%s: Dear %s, your order #%s has been %s.', 'sendeasy'),
                        '{shop_name}',
                        '{billing_first_name}',
                        '{order_id}',
                        '{order_status}'
                    ),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_customer_sms_template_completed',
                    'name' => __('Order completed SMS template', 'sendeasy'),
                    'desc_tip' => __('Customise the SMS message sent for this order status. If left blank, the default SMS message will be used.', 'sendeasy'),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'textarea',
                    'id'        => 'se_customer_sms_template_pending',
                    'name'      => __('Order pending payment SMS template', 'sendeasy'),
                    'desc_tip' => __('Customise the SMS message sent for this order status. If left blank, the default SMS message will be used.', 'sendeasy'),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_customer_sms_template_processing',
                    'name' => __('Order processing SMS template', 'sendeasy'),
                    'desc_tip' => __('Customise the SMS message sent for this order status. If left blank, the default SMS message will be used.', 'sendeasy'),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_customer_sms_template_on-hold',
                    'name' => __('Order on-hold SMS template', 'sendeasy'),
                    'desc_tip' => __('Customise the SMS message sent for this order status. If left blank, the default SMS message will be used.', 'sendeasy'),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_customer_sms_template_cancelled',
                    'name' => __('Order cancelled SMS template', 'sendeasy'),
                    'desc_tip' => __('Customise the SMS message sent for this order status. If left blank, the default SMS message will be used.', 'sendeasy'),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_customer_sms_template_failed',
                    'name' => __('Order failed SMS template', 'sendeasy'),
                    'desc_tip' => __('Customise the SMS message sent for this order status. If left blank, the default SMS message will be used.', 'sendeasy'),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_customer_sms_template_refunded',
                    'name' => __('Order refunded SMS template', 'sendeasy'),
                    'desc_tip'  => __('Customise the SMS message sent for this order status. If left blank, the default SMS message will be used.', 'sendeasy'),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'sectionend',
                    'id' => 'se_sendeasy_customer_sms_options'
                ],

            ]);
        } elseif ('se_admin_sms_section' == $current_section) {

            /**
             * Filter the SendEasy admin SMS section settings
             *
             * @param array $settings
             */
            $settings = apply_filters('se_admin_sms_section_settings', [
                [
                    'name' => __('Admin SMS notifications', 'sendeasy'),
                    'type' => 'title',
                    'id' => 'se_admin_sms_options',
                ],
                [
                    'type' => 'checkbox',
                    'id' => 'se_enable_admin_sms',
                    'name' => __('Enable new order admin SMS notifications', 'sendeasy'),
                    'desc' => __('If enabled, SMS notifications will be sent to admin cell phone number(s).', 'sendeasy'),
                    'default'  => 'no'
                ],
                [
                    'type'  => 'text',
                    'id' => 'se_admin_phone_numbers',
                    'placeholder' => '27825657891',
                    'name' => __('Admin cell phone number(s) ', 'sendeasy'),
                    'desc_tip' => __('Enter cell phone number(s) to send new order notifications to. Send notifications to multiple numbers by separating them with a comma. Include the "27" country code for South Africa. Eg., "27825657891"', 'sendeasy'),
                ],
                [
                    'type' => 'textarea',
                    'id' => 'se_admin_sms_template',
                    'name' => __('Admin SMS template', 'sendeasy'),
                    'desc' => sprintf(
                        '%s
                                <ul>
                                    <li><code>{shop_name}</code>: <i>%s</i></li>
                                    <li><code>{order_id}</code>: <i>%s</i></li>
                                    <li><code>{order_status}</code>: <i>%s</i></li>
                                    <li><code>{order_total}</code>: <i>%s</i></li>
                                    <li><code>{billing_first_name}</code>: <i>%s</i></li>
                                    <li><code>{billing_last_name}</code>: <i>%s</i></li>
                                    <li><code>{shipping_first_name}</code>: <i>%s</i></li>
                                    <li><code>{shipping_last_name}</code>: <i>%s</i></li>
                                    <li><code>{shipping_method}</code>: <i>%s</i></li>
                                </ul>
                                <b><em>%s</em></b>',
                        __('Use these tags to customise your message:', 'sendeasy'),
                        __('Your WordPress site title.', 'sendeasy'),
                        __("Customer's order number.", 'sendeasy'),
                        __("Customer's order status.", 'sendeasy'),
                        __("Total cost of the customer's order.", 'sendeasy'),
                        __("Customer's first name.", 'sendeasy'),
                        __("Customer's last name.", 'sendeasy'),
                        __("Recipient's first name, if shipping to a different address.", 'sendeasy'),
                        __("Recipient's last name, if shipping to a different address.", 'sendeasy'),
                        __('Shipping method chosen for the order.', 'sendeasy'),
                        __('Remember that SMS messages must be limited to 160 characters or less.', 'sendeasy')
                    ),
                    'default' => sprintf(
                        __('%s: Dear Admin, you have an a new order #%s for the amount of %s. Congratulations!', 'sendeasy'),
                        '{shop_name}',
                        '{order_id}',
                        '{order_total}'
                    ),
                    'css' => 'min-width: 50%; height: 75px;'
                ],
                [
                    'type' => 'sectionend',
                    'id' => 'se_admin_sms_options'
                ],
            ]);
        } else {

            /**
             * Filter SendEasy connection section settings
             *
             * @param array $settings
             */
            $settings = apply_filters('se_sendeasy_connection_section_settings', [

                [
                    'name' => __('Connection settings', 'sendeasy'),
                    'type' => 'title',
                    'desc' => __('Ensure that your SendEasy API credentials are entered below to use the SendEasy plugin.', 'sendeasy'),
                    'id' => 'se_sendeasy_connection_options'
                ],
                [
                    'type' => 'text',
                    'id' => 'se_username',
                    'name' => __('Username', 'sendeasy'),
                    'desc_tip' => __('Enter your username here to connect to the SendEasy API.', 'sendeasy'),
                ],
                [
                    'type' => 'text',
                    'id' => 'se_password',
                    'name' => __('Password', 'sendeasy'),
                    'desc_tip' => __('Enter your password here to connect to the SendEasy API.', 'sendeasy'),
                ],
                [
                    'id'  => 'se_enable_debug_log',
                    'name' => __('Log errors', 'sendeasy'),
                    'desc' => sprintf(__('Enable this to record SendEasy errors in the %s log. Use this if you are having issues when trying to send an SMS.', 'sendeasy'), 'WooCommerce'),
                    'type' => 'checkbox',
                    'default' => 'no'
                ],
                [
                    'type' => 'sectionend',
                    'id' => 'se_sendeasy_connection_options'
                ],
            ]);
        }

        /**
         * Filter the SendEasy settings
         *
         * @param array $settings
         */
        return apply_filters('woocommerce_get_settings_' . $this->id, $settings, $current_section);
    }


    /**
     * Output the SendEasy settings
     *
     * @access public
     * @return void
     */
    public function output(): void
    {
        global $current_section;
        $settings = $this->get_settings($current_section);

        $this->se_output_sms_credit_balance_notice();
        $this->se_output_sendeasy_registration_instructions($current_section);

        WC_Admin_Settings::output_fields($settings);
    }


    /**
     * Save the SendEasy settings
     *
     * @access public
     * @return void
     */
    public function save(): void
    {
        global $current_section;
        $settings = $this->get_settings($current_section);
        WC_Admin_Settings::save_fields($settings);
    }

    /**
     * Outputs the SendEasy SMS credit balance notice
     *
     * @access private
     * @return void
     *
     */
    private function se_output_sms_credit_balance_notice(): void
    {
        $username = get_option('se_username');
        $password = get_option('se_password');

        if (!empty($username) && !empty($password)) {

            $response = SE_SOAP_Client::se_query_credit_balance([
                'fields' => [
                    'uID' => $username,
                    'pwd' => $password
                ]
            ]);

            extract($response);

            if ($response_status === 'success') {
                include SE_SENDEASY_PLUGIN_DIR_PATH . 'templates/sendeasy-sms-credit-balance.php';
            } else {
                include SE_SENDEASY_PLUGIN_DIR_PATH . 'templates/sendeasy-wc-error-notice.php';
            }
            include SE_SENDEASY_PLUGIN_DIR_PATH . 'templates/sendeasy-buy-more-credit-notice.php';
        }
    }

    /**
     * Outputs the SendEasy registration instructions
     *
     * @access private
     * @param string $current_section
     * @return void
     */
    private function se_output_sendeasy_registration_instructions(string $current_section): void
    {
        if ($current_section !== 'se_customer_sms_section' && $current_section !== 'se_admin_sms_section') {

            $username = get_option('se_username');
            $password = get_option('se_password');

            if (empty($username) || empty($password)) {
                include SE_SENDEASY_PLUGIN_DIR_PATH . 'templates/sendeasy-registration-instructions.php';
            }
        }
    }
}

return new SE_Sendeasy_WC_Settings_Page();
